"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var create_item_exports = {};
__export(create_item_exports, {
  createItemTemplate: () => createItemTemplate,
  makeKey: () => makeKey
});
module.exports = __toCommonJS(create_item_exports);
var import_helpers = require("../../helpers");
var import_computed_fields = require("../computed-fields");
var import_ensure_table = require("./ensure-table");
var import_helpers2 = require("./helpers");
var import_indexes = require("./indexes");
function createItemTemplate(config, model) {
  const {
    fields,
    isPublic: hasPublicId,
    primaryKey,
    table: { tableName },
    ttlConfig,
    typeName
  } = model;
  const key = makeKey(primaryKey, config);
  const inputTypeName = `Create${typeName}Input`;
  const omitInputFields = [
    "createdAt",
    "id",
    "updatedAt",
    "version",
    hasPublicId && "publicId",
    ttlConfig?.fieldName,
    ...fields.filter(({ computeFunction }) => !!computeFunction).map(({ fieldName }) => fieldName)
  ].filter(import_helpers.filterNull).map((f) => `'${f}'`).sort();
  const outputTypeName = `Create${typeName}Output`;
  return `
export type ${inputTypeName} = Omit<${typeName}, ${omitInputFields.join("|")}>${ttlConfig ? ` & Partial<Pick<${typeName}, '${ttlConfig.fieldName}'>>` : ""};
export type ${outputTypeName} = ResultType<${typeName}>
/**  */
export async function create${typeName}(${(0, import_computed_fields.inputName)(
    model
  )}: Readonly<Create${typeName}Input>): Promise<Readonly<${outputTypeName}>> {
${(0, import_ensure_table.ensureTableTemplate)(tableName)}

  const now = new Date();

  ${(0, import_computed_fields.defineComputedInputFields)(fields, typeName)}
  const {ExpressionAttributeNames, ExpressionAttributeValues, UpdateExpression} = marshall${typeName}(input, now);

  ${hasPublicId ? `const publicId = idGenerator();` : ""}
  try {
    // Reminder: we use UpdateCommand rather than PutCommand because PutCommand
    // cannot return the newly written values.
    const commandInput: UpdateCommandInput = {
      ConditionExpression: 'attribute_not_exists(#pk)',
      ExpressionAttributeNames: {
        ...ExpressionAttributeNames,
        '#createdAt': '_ct',
        ${hasPublicId ? "'#publicId': 'publicId'," : ""}
            ${model.secondaryIndexes.filter(
    (index) => (0, import_indexes.indexHasField)("createdAt", model.primaryKey, index)
  ).map((index) => (0, import_indexes.indexToEANPart)(config, index)).flat().join("\n")}
      },
      ExpressionAttributeValues: {
        ...ExpressionAttributeValues,
        ':createdAt': now.getTime(),
        ${hasPublicId ? "':publicId': publicId," : ""}
            ${model.secondaryIndexes.filter(
    (index) => (0, import_indexes.indexHasField)("createdAt", model.primaryKey, index)
  ).map((index) => (0, import_indexes.indexToEAVPart)("create", index)).flat().join("\n")}
      },
      Key: ${(0, import_helpers2.objectToString)(key)},
      ReturnConsumedCapacity: 'INDEXES',
      ReturnItemCollectionMetrics: 'SIZE',
      ReturnValues: 'ALL_NEW',
      TableName: tableName,
      UpdateExpression: [
        ...UpdateExpression.split(', '),
        '#createdAt = :createdAt',
        ${hasPublicId ? "'#publicId = :publicId'," : ""}
        ${model.secondaryIndexes.filter(
    (index) => (0, import_indexes.indexHasField)("createdAt", model.primaryKey, index)
  ).map(import_indexes.indexToUpdateExpressionPart).flat().join("\n")}
      ].join(', ')
    };

    const {ConsumedCapacity: capacity, ItemCollectionMetrics: metrics, Attributes: item} = await ddbDocClient.send(new UpdateCommand(commandInput));

    assert(capacity, 'Expected ConsumedCapacity to be returned. This is a bug in codegen.');

    assert(item, 'Expected DynamoDB to return an Attributes prop.');
    assert(item._et === '${typeName}', () => new DataIntegrityError(\`Expected to write ${typeName} but wrote \${item?._et} instead\`));

    return {
      capacity,
      item: unmarshall${typeName}(item),
      metrics,
    }
  }
  catch (err) {
    if (err instanceof ConditionalCheckFailedException) {
      throw new AlreadyExistsError('${typeName}', ${(0, import_helpers2.objectToString)(key)});
    }
    ${(0, import_helpers2.handleCommonErrors)()}
  }
}`;
}
function makeKey(key, config) {
  if (key.isComposite) {
    const doLegacy = config.legacyEmptySortFieldBehavior && key.sortKeyFields.length === 0;
    return {
      pk: (0, import_helpers2.makeKeyTemplate)(
        key.partitionKeyPrefix,
        key.partitionKeyFields,
        "create"
      ),
      sk: doLegacy ? `'${key.sortKeyPrefix}#0'` : (0, import_helpers2.makeKeyTemplate)(key.sortKeyPrefix, key.sortKeyFields, "create")
    };
  }
  return {
    pk: (0, import_helpers2.makeKeyTemplate)(
      key.partitionKeyPrefix,
      key.partitionKeyFields,
      "create"
    )
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createItemTemplate,
  makeKey
});
