"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var marshall_exports = {};
__export(marshall_exports, {
  marshallTpl: () => marshallTpl
});
module.exports = __toCommonJS(marshall_exports);
var import_assert = require("@code-like-a-carpenter/assert");
var import_helpers = require("../../helpers");
var import_helpers2 = require("./helpers");
var import_indexes = require("./indexes");
function wrapFieldNameWithQuotes({ fieldName }) {
  return `'${fieldName}'`;
}
function makeTypeDefinition(typeName, requiredFields, optionalFields) {
  const rf = requiredFields.map(wrapFieldNameWithQuotes).sort().join("|");
  const of = optionalFields.map(wrapFieldNameWithQuotes).sort().join("|");
  let marshallType = `Required<Pick<${typeName}, ${rf}>>`;
  if (of.length) {
    marshallType += ` & Partial<Pick<${typeName}, ${of}>>`;
  }
  return marshallType;
}
function marshallTpl(config, { fields, primaryKey, secondaryIndexes, ttlConfig, typeName }) {
  const requiredFields = fields.filter((f) => f.isRequired && f.fieldName !== "publicId").filter(({ fieldName }) => fieldName !== "id");
  const optionalFields = fields.filter(
    (f) => !f.isRequired && f.fieldName !== "publicId"
  );
  const requiredFieldsWithDefaultBehaviorsNames = [
    "version",
    ttlConfig?.fieldName
  ].filter(import_helpers.filterNull);
  const requiredFieldsWithDefaultBehaviors = requiredFields.filter(
    ({ fieldName }) => requiredFieldsWithDefaultBehaviorsNames.includes(fieldName)
  );
  const builtinDateFieldNames = ["createdAt", "updatedAt"];
  const normalRequiredFields = requiredFields.filter(
    ({ fieldName }) => !requiredFieldsWithDefaultBehaviorsNames.includes(fieldName) && !builtinDateFieldNames.includes(fieldName)
  );
  const marshallType = makeTypeDefinition(typeName, normalRequiredFields, [
    ...optionalFields,
    ...requiredFieldsWithDefaultBehaviors
  ]);
  const inputTypeName = `Marshall${typeName}Input`;
  return `
export interface Marshall${typeName}Output {
  ExpressionAttributeNames: Record<string, string>;
  ExpressionAttributeValues: Record<string, NativeAttributeValue>;
  UpdateExpression: string;
}

export type ${inputTypeName} = ${marshallType};


/** Marshalls a DynamoDB record into a ${typeName} object */
export function marshall${typeName}(input: ${inputTypeName}, now = new Date()): Marshall${typeName}Output {
  const updateExpression: string[] = [
  "#entity = :entity",
  ${requiredFields.filter(
    ({ fieldName }) => fieldName !== ttlConfig?.fieldName && fieldName !== "createdAt"
  ).map(({ fieldName }) => `'#${fieldName} = :${fieldName}',`).join("\n")}
  ${secondaryIndexes.filter(({ name }) => name !== "publicId").filter((index) => !(0, import_indexes.indexHasField)("createdAt", primaryKey, index)).map(import_indexes.indexToUpdateExpressionPart).flat().join("\n")}
  ];

  const ean: Record<string, string> = {
    "#entity": "_et",
    "#pk": "pk",
${requiredFields.filter(
    ({ fieldName }) => fieldName !== ttlConfig?.fieldName && fieldName !== "createdAt"
  ).map(({ columnName, fieldName }) => `'#${fieldName}': '${columnName}',`).join("\n")}
${secondaryIndexes.filter(({ name }) => name !== "publicId").filter((index) => !(0, import_indexes.indexHasField)("createdAt", primaryKey, index)).map((index) => (0, import_indexes.indexToEANPart)(config, index)).flat().join("\n")}
  };

  const eav: Record<string, unknown> = {
    ":entity": "${typeName}",
    ${normalRequiredFields.map((field) => `':${field.fieldName}': ${(0, import_helpers2.marshallField)(field)},`).join("\n")}
    ':updatedAt': now.getTime(),
    ${requiredFieldsWithDefaultBehaviors.map(({ fieldName }) => {
    if (fieldName === "version") {
      return `':version': ('version' in input ? (input.version ?? 0) : 0) + 1,`;
    }
    if (fieldName === ttlConfig?.fieldName) {
      return "";
    }
    throw new Error(`No default behavior for field \`${fieldName}\``);
  }).filter(import_helpers.filterNull).join("\n")}
${secondaryIndexes.filter(({ name }) => name !== "publicId").filter((index) => !(0, import_indexes.indexHasField)("createdAt", primaryKey, index)).map((index) => (0, import_indexes.indexToEAVPart)("read", index)).flat().join("\n")}
  };

  ${optionalFields.filter(({ fieldName }) => fieldName !== ttlConfig?.fieldName).filter(({ computeFunction }) => !computeFunction?.isVirtual).map(
    (field) => `
  if ('${field.fieldName}' in input && typeof input.${field.fieldName} !== 'undefined') {
    ean['#${field.fieldName}'] = '${field.columnName}';
    eav[':${field.fieldName}'] = ${(0, import_helpers2.marshallField)(field)};
    updateExpression.push('#${field.fieldName} = :${field.fieldName}');
  }
  `
  ).join("\n")};

  ${renderTTL(ttlConfig, fields)}

  updateExpression.sort();

  return {
    ExpressionAttributeNames: ean,
    ExpressionAttributeValues: eav,
    UpdateExpression: "SET " + updateExpression.join(", ")
  };
}

  `;
}
function renderTTL(ttlConfig, fields) {
  if (!ttlConfig) {
    return "";
  }
  const { argumentAllowed, argumentRequired, fieldName } = ttlConfig;
  const field = fields.find(({ fieldName: f }) => f === fieldName);
  (0, import_assert.assert)(field, `Field ${fieldName} not found`);
  if (argumentRequired) {
    return `
      assert(!Number.isNaN(input.${fieldName}.getTime()),'${fieldName} was passed but is not a valid date')
      ean['#${fieldName}'] = 'ttl';
      eav[':${fieldName}'] = ${(0, import_helpers2.marshallField)(field)};
      updateExpression.push('#${fieldName} = :${fieldName}');
    `;
  }
  const defaultValue = "duration" in ttlConfig ? `now.getTime() + ${ttlConfig.duration}` : "now.getTime()";
  if (!argumentAllowed) {
    return `
      ean['#${fieldName}'] = 'ttl';
      eav[':${fieldName}'] = Math.floor((${defaultValue})/1000);
      updateExpression.push('#${fieldName} = :${fieldName}');
    `;
  }
  return `
    ean['#${fieldName}'] = 'ttl';
    if ('${fieldName}' in input && typeof input.${fieldName} !== 'undefined' && input.${fieldName} !== null) {
      assert(!Number.isNaN(input.${fieldName}.getTime()),'${fieldName} was passed but is not a valid date')
      eav[':${fieldName}'] = ${(0, import_helpers2.marshallField)(field)};
    }
    else {
      eav[':${fieldName}'] = Math.floor((${defaultValue})/1000);
    }
    updateExpression.push('#${fieldName} = :${fieldName}');
  `;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  marshallTpl
});
